// Copyright (c)2002-2003 Eric Carlson
// This source is licensed under the BSD license.
// Please see the LICENSE.txt file included with the distribution for more information.

#include "RezChangeNotification.h"

// this global refers to our notification callback
DMExtendedNotificationUPP gDMNotifyUPP = NULL;

//globals for current display state
//used in callback to compare with new values to see if display mode changed
long current_width = 0;
long current_height = 0;
long current_refresh = 0;

//set up the display mode notification callback with the Display Manager
void registerNotification(OurDMNotifyDataPtr dataP)
{
    ProcessSerialNumber	currentPSN;
    CFDictionaryRef newMode = NULL;
    CFNumberRef number = NULL;

    if( (newMode = CGDisplayCurrentMode(kCGDirectMainDisplay)) != NULL) {
        number = CFDictionaryGetValue(newMode, kCGDisplayWidth);
        CFNumberGetValue( number, kCFNumberLongType, &current_width ) ;

        number =  CFDictionaryGetValue(newMode, kCGDisplayHeight);
        CFNumberGetValue( number, kCFNumberLongType, &current_height ) ;

        number = CFDictionaryGetValue(newMode, kCGDisplayRefreshRate);
        CFNumberGetValue( number, kCFNumberLongType, &current_refresh ) ;        
    }


    GetCurrentProcess( &currentPSN );
    gDMNotifyUPP = NewDMExtendedNotificationUPP( notificationCallback );
    DMRegisterExtendedNotifyProc( gDMNotifyUPP, (void*) dataP, (unsigned short) NULL, &currentPSN );    
}

//remove our callback
void removeNotification(OurDMNotifyDataPtr dataP)
{
    ProcessSerialNumber	currentPSN;
    
    // Remove Display Manager's notification proc
    GetCurrentProcess( &currentPSN );
    DMRemoveExtendedNotifyProc( gDMNotifyUPP, (void*) dataP, &currentPSN, (unsigned short) NULL);
    DisposeRoutineDescriptor( gDMNotifyUPP );
}

//the callback to be called by the display manager when a display change occurs
pascal void notificationCallback (void* userData, short theMessage, void* notifyData)
{
    CFDictionaryRef newMode = NULL;
    CFNumberRef number = NULL;
    long width;
    long height;
    long refresh;

    //information needed to call the objective-c update selector
    OurDMNotifyDataPtr dataP = (OurDMNotifyDataPtr) userData;

    //if it's not a notification event, we don't care
    //important since each resolution change results in two notifications.
    //we use this to choose one
    if ( theMessage != kDMNotifyEvent ) {
        return;
    }

    //get information about the current mode to compare with what we had previously
    //use this to determine if the monitor's mode changed (not just color depth) and
    //thus we need to grab the display parameter values for this mode from the monitor
    if( (newMode = CGDisplayCurrentMode(kCGDirectMainDisplay)) == NULL) {
        return;
    }
    
    number = CFDictionaryGetValue(newMode, kCGDisplayWidth);
    CFNumberGetValue( number, kCFNumberLongType, &width ) ;

    number =  CFDictionaryGetValue(newMode, kCGDisplayHeight);
    CFNumberGetValue( number, kCFNumberLongType, &height ) ;
    
    number = CFDictionaryGetValue(newMode, kCGDisplayRefreshRate);
    CFNumberGetValue( number, kCFNumberLongType, &refresh ) ;

    //if width, height, or refresh rate changed, inform controller so it can grab new settings and update GUI
    if( (width!=current_width) || (height!=current_height) || (refresh!=current_refresh) ) {
        sleep(1); //wait for monitor settings to stabilize before re-reading them
        dataP->updateMethod(dataP->updateMethodSelf, dataP->updateMethodSelector);
        current_width=width;
        current_height=height;
        current_refresh=refresh;
    }
    
    return;
}