// Copyright (c)2002-2003 Eric Carlson
// This source is licensed under the BSD license.
// Please see the LICENSE.txt file included with the distribution for more information.

#import <Cocoa/Cocoa.h>
#import <ApplicationServices/ApplicationServices.h>
#import "USBMonitor.h"
#import "TestPatternView.h"

//for sleep()
#include <unistd.h>

//tags associated with the various geometry pane radio buttons
const int geometryElementTags[] = {
    kHIDVESAUsagePage_HorizPin,
    kHIDVESAUsagePage_HorizPinBal,
    kHIDVESAUsagePage_HorizLin,
    kHIDVESAUsagePage_HorizLinBal,
    kHIDVESAUsagePage_VertPin,
    kHIDVESAUsagePage_VertPinBal,
    kHIDVESAUsagePage_VertLin,
    kHIDVESAUsagePage_VertLinBal,
    kHIDVESAUsagePage_KeyBalance,
    kHIDVESAUsagePage_Key,
    kHIDVESAUsagePage_Tilt,
    kHIDVESAUsagePage_TCDistControl,
    kHIDVESAUsagePage_TCDistBalance,
    kHIDVESAUsagePage_BCDistControl,
    kHIDVESAUsagePage_BCDistBalance,
    kHIDVESAUsagePage_Focus,
    kHIDVESAUsagePage_HorizMoire,
    kHIDVESAUsagePage_VertMoire
};

@interface SliderController : NSObject
{
    //Screen tab
    IBOutlet id brightnessSlider;
    IBOutlet id contrastSlider;
    IBOutlet id horizontalPositionSlider;
    IBOutlet id horizontalSizeSlider;
    IBOutlet id verticalPositionSlider;
    IBOutlet id verticalSizeSlider;
    IBOutlet id brightnessValue;
    IBOutlet id contrastValue;
    IBOutlet id horizontalPositionValue;
    IBOutlet id horizontalSizeValue;
    IBOutlet id verticalPositionValue;
    IBOutlet id verticalSizeValue;
    IBOutlet id brightnessStepper;
    IBOutlet id contrastStepper;
    IBOutlet id horizontalPositionStepper;
    IBOutlet id horizontalSizeStepper;
    IBOutlet id verticalPositionStepper;
    IBOutlet id verticalSizeStepper;

    //color tab
    IBOutlet id redGainSlider;
    IBOutlet id redGainValue;
    IBOutlet id redGainStepper;
    IBOutlet id greenGainSlider;
    IBOutlet id greenGainValue;
    IBOutlet id greenGainStepper;
    IBOutlet id blueGainSlider;
    IBOutlet id blueGainValue;
    IBOutlet id blueGainStepper;
    IBOutlet id redLuminanceSlider;
    IBOutlet id redLuminanceValue;
    IBOutlet id redLuminanceStepper;
    IBOutlet id greenLuminanceSlider;
    IBOutlet id greenLuminanceValue;
    IBOutlet id greenLuminanceStepper;
    IBOutlet id blueLuminanceSlider;
    IBOutlet id blueLuminanceValue;
    IBOutlet id blueLuminanceStepper;
    
    //geometry tab
    IBOutlet id geometrySlider;
    IBOutlet id geometrySelector;
    IBOutlet id geometryValue;
    IBOutlet id geometryStepper;

    //advanced tab
    IBOutlet id horizontalMisconvergenceSlider;
    IBOutlet id horizontalMisconvergenceValue;
    IBOutlet id horizontalMisconvergenceStepper;
    IBOutlet id verticalMisconvergenceSlider;
    IBOutlet id verticalMisconvergenceValue;
    IBOutlet id verticalMisconvergenceStepper;
    IBOutlet id degaussButton;
    
    IBOutlet NSWindow *revertProgressSheet;
    IBOutlet NSWindow *mainWindow;
    NSWindow *fullScreenWindow;
    TestPatternView *patterns;

    IBOutlet id hideShowControlsWindowMenuItem;

    IBOutlet id revertProgress;

    //menu item to which we attach a submenu allowing selection of which monitor to control
    IBOutlet id targetMonitorSelectMenu;
    
    USBMonitor *myMonitor; //the object representing the monitor
    float *originalSettings; //the settings array grabbed at startup

    //"current" screen params used to detect a change in resolution/refresh
    long current_width, current_height, current_refresh;

    //dictionary of NSArrays, one for each "group" of associated controls
    NSMutableDictionary *controlGroups;
}

//change the monitor being controlled
- (IBAction)changeTargetMonitor:(id)sender;

//called by UI elements
- (IBAction)degauss:(id)sender;
- (IBAction)updateGeometrySliderTarget:(id)sender;
- (IBAction)changeValue:(id)sender;

- (IBAction)showTestPatterns:(id)sender;
- (IBAction)hideTestPatterns:(id)sender;
- (void)showControlsWindow:(id)sender;

//helpers
- (void)updateControlValues;
- (void)initControlTags;
- (void)initControlGroups;
- (void)disableUnimplementedControls;

//callback selector for no monitor error sheet
- (void)noMonitorSheetDidEnd:(NSWindow *)sheet
                  returnCode:(int)returnCode
                 contextInfo:(void *)contextInfo;

//methods for dealing with revert sheets
- (void)updateRevertProgress;
- (void)raiseRevertProgressSheet;
- (void)endRevertProgressSheet;
- (void)revertProgressSheetDidEnd:(NSWindow *)sheet
                       returnCode:(int)returnCode
                      contextInfo:(void *)contextInfo;
- (IBAction)revertSettings:(id)sender;
- (void)revertSheetDidDismiss:(NSWindow *)sheet
                   returnCode:(int)returnCode
                  contextInfo:(void*)contextInfo;
@end
