// Copyright (c)2002-2003 Eric Carlson
// This source is licensed under the BSD license.
// Please see the LICENSE.txt file included with the distribution for more information.

#import <Foundation/Foundation.h>
#import "HID_Utilities_External.h"
#import "USBMonitorControlClassSpec.h"

//maximum number of supported monitors we'll support (arbitrary, but unlikely to be exceeded)
#define MAX_MONITORS 8

//this is overkill since the USB Monitor Control Class Spec for VESA controls
//doesn't have usage values up to 0xFF, but just to be safe into the future (it's a very sparse array to begin with)
#define MYELEMENTS_LENGTH 255

@interface USBMonitor : NSObject
{
    //these arrays could be dynamically allocated I suppose... but it's hardly worth the effort
    pRecDevice monitorList[MAX_MONITORS]; //pointers to all supported monitors found
    pRecElement* monitorElementsList[MAX_MONITORS]; //pointers to each supported monitor's elements list
    
    pRecElement* myElements; //HID elements for myMonitor
    pRecDevice myMonitor; //pointer to the currently selected monitor
    int selectedMonitor; //index of the currently selected monitor
    int numberOfMonitors; //total number of supported monitors found
}

//methods to support multiple monitors
- (int)getNumberOfMonitors;
- (int)selectedMonitor;
- (BOOL)changeSelectedMonitorTo:(int)value;

//set values
- (BOOL)setBrightness:(float)value;
- (BOOL)setContrast:(float)value;
- (BOOL)setRedGain:(float)value;
- (BOOL)setGreenGain:(float)value;
- (BOOL)setBlueGain:(float)value;
- (BOOL)setFocus:(float)value;
- (BOOL)setHorizontalPosition:(float)value;
- (BOOL)setHorizontalSize:(float)value;
- (BOOL)setHorizontalPincushion:(float)value;
- (BOOL)setHorizontalPincushionBalance:(float)value;
- (BOOL)setHorizontalMisconvergence:(float)value;
- (BOOL)setHorizontalLinearity:(float)value;
- (BOOL)setHorizontalLinearityBalance:(float)value;
- (BOOL)setVerticalPosition:(float)value;
- (BOOL)setVerticalSize:(float)value;
- (BOOL)setVerticalPincushion:(float)value;
- (BOOL)setVerticalPincushionBalance:(float)value;
- (BOOL)setVerticalMisconvergence:(float)value;
- (BOOL)setVerticalLinearity:(float)value;
- (BOOL)setVerticalLinearityBalance:(float)value;
- (BOOL)setKeystoneBalance:(float)value;
- (BOOL)setKeystone:(float)value;
- (BOOL)setRotation:(float)value;
- (BOOL)setTopCornerDistortionControl:(float)value;
- (BOOL)setTopCornerDistortionBalance:(float)value;
- (BOOL)setBottomCornerDistortionControl:(float)value;
- (BOOL)setBottomCornerDistortionBalance:(float)value;
- (BOOL)setHorizontalMoire:(float)value;
- (BOOL)setVerticalMoire:(float)value;
- (BOOL)setRedLuminance:(float)value;
- (BOOL)setGreenLuminance:(float)value;
- (BOOL)setBlueLuminance:(float)value;
- (BOOL)degauss;


//check if the monitor supports a control
- (BOOL)hasBrightness;
- (BOOL)hasContrast;
- (BOOL)hasRedGain;
- (BOOL)hasGreenGain;
- (BOOL)hasBlueGain;
- (BOOL)hasFocus;
- (BOOL)hasHorizontalPosition;
- (BOOL)hasHorizontalSize;
- (BOOL)hasHorizontalPincushion;
- (BOOL)hasHorizontalPincushionBalance;
- (BOOL)hasHorizontalMisconvergence;
- (BOOL)hasHorizontalLinearity;
- (BOOL)hasHorizontalLinearityBalance;
- (BOOL)hasVerticalPosition;
- (BOOL)hasVerticalSize;
- (BOOL)hasVerticalPincushion;
- (BOOL)hasVerticalPincushionBalance;
- (BOOL)hasVerticalMisconvergence;
- (BOOL)hasVerticalLinearity;
- (BOOL)hasVerticalLinearityBalance;
- (BOOL)hasKeystoneBalance;
- (BOOL)hasKeystone;
- (BOOL)hasRotation;
- (BOOL)hasTopCornerDistortionControl;
- (BOOL)hasTopCornerDistortionBalance;
- (BOOL)hasBottomCornerDistortionControl;
- (BOOL)hasBottomCornerDistortionBalance;
- (BOOL)hasHorizontalMoire;
- (BOOL)hasVerticalMoire;
- (BOOL)hasRedLuminance;
- (BOOL)hasGreenLuminance;
- (BOOL)hasBlueLuminance;
- (BOOL)hasDegauss;


//get current values
- (float)brightness;
- (float)contrast;
- (float)redGain;
- (float)greenGain;
- (float)blueGain;
- (float)focus;
- (float)horizontalPosition;
- (float)horizontalSize;
- (float)horizontalPincushion;
- (float)horizontalPincushionBalance;
- (float)horizontalMisconvergence;
- (float)horizontalLinearity;
- (float)horizontalLinearityBalance;
- (float)verticalPosition;
- (float)verticalSize;
- (float)verticalPincushion;
- (float)verticalPincushionBalance;
- (float)verticalMisconvergence;
- (float)verticalLinearity;
- (float)verticalLinearityBalance;
- (float)keystoneBalance;
- (float)keystone;
- (float)rotation;
- (float)topCornerDistortionControl;
- (float)topCornerDistortionBalance;
- (float)bottomCornerDistortionControl;
- (float)bottomCornerDistortionBalance;
- (float)horizontalMoire;
- (float)verticalMoire;
- (float)redLuminance;
- (float)greenLuminance;
- (float)blueLuminance;


//methods to handle bulk actions
- (void)updateSettingsUsingArray:(float*)settings
            withProgressSelector:(SEL)updateProgress
                selectorWhenDone:(SEL)finish
                         ofClass:(NSObject *)theClass;
- (float*)settingsArray;

//private methods used by above methods
- (float)getHIDElementValue:(int)key;
- (BOOL)hasHIDElementWithKey:(int)key;
- (BOOL)setHIDElement:(int)key withValue:(float)value;

@end
