// Copyright (c)2002-2003 Eric Carlson
// This source is licensed under the BSD license.
// Please see the LICENSE.txt file included with the distribution for more information.

#import "USBMonitor.h"

@implementation USBMonitor

//rewrite this using CFDictionary instead of a raw array (sometime)
- (id)init
{
    BOOL found = FALSE;
    pRecDevice theDevice;
    pRecElement anElement;
//    int deviceCount=0;
    int monitorsIndex = 0;
    
    self = [super init];

    //get the device list so we can iterate through it
    HIDBuildDeviceList(0,0);

    //NSLog(@"**************** Diamond Control USB: Device List *******************\n");
    //iterate through device list looking for a supported monitor
    theDevice = HIDGetFirstDevice();
    while(theDevice)
    {
        /*
        NSLog(@"Device %d found:\n",deviceCount++);
        NSLog(@"productID: %ld\n",theDevice->productID);
        NSLog(@"vendorID: %ld\n",theDevice->vendorID);
        NSLog(@"Manufacturer: %s\n",theDevice->manufacturer);
        NSLog(@"Product: %s\n",theDevice->product);
        NSLog(@"usage: 0x%lx\n",theDevice->usage);
        NSLog(@"usagePage: 0x%lx\n",theDevice->usagePage);
        NSLog(@"totalElements: 0x%lx\n",theDevice->totalElements);
        NSLog(@"\n");
         */
        if( theDevice->usagePage == kHIDPage_Monitor )
        {
            //we found a monitor, so get references to all its elements

            //allocate memory for the elements array for this device
            //(yes, we really do need to zero all the pointers...)
            monitorElementsList[monitorsIndex] = (pRecElement*) calloc(MYELEMENTS_LENGTH,sizeof(pRecElement));
            
            anElement = HIDGetFirstDeviceElement(theDevice,kHIDElementTypeAll);
            while(anElement)
            {
                if(anElement->usage > MYELEMENTS_LENGTH-1)
                {
                    NSLog(@"Element usage value out of range: %d > %d",anElement->usage, sizeof(myElements)-1);
                    return nil;
                }

                if(anElement->usagePage == kHIDPage_VESA)
                {
                    monitorElementsList[monitorsIndex][anElement->usage] = anElement; //put it in the elements array for this monitor
                    /*
                    NSLog(@"min: %ld, max: %ld\n",anElement->min, anElement->max);
                    NSLog(@"scaledMin: %ld, scaledMax: %ld\n",anElement->scaledMin, anElement->scaledMax);
                    NSLog(@"calMin: %ld, calMax: %ld\n",anElement->calMin, anElement->calMax);
                    NSLog(@"userMin: %ld, userMax: %ld\n",anElement->userMin, anElement->userMax);
                     */
                }
                
                anElement = HIDGetNextDeviceElement(anElement,kHIDElementTypeAll);
            }
            found = TRUE; //mark that we found at least one supported device
            monitorList[monitorsIndex++] = theDevice; //put this device in our list of found displays
            if(monitorsIndex >= MAX_MONITORS)
            {
                NSLog(@"You appear to have more than %d supported displays connected to this system! Please let the author know so he can envy you (and update the software).\n",MAX_MONITORS);
                break; //too many monitors found, so stop
            }
        } //end check if this is the device we want

        theDevice = HIDGetNextDevice(theDevice);
    } //end device search
    
    //NSLog(@"**************** END Diamond Control USB: Device List *******************\n");
    
    if(found)
    {
        selectedMonitor = 0;
        numberOfMonitors = monitorsIndex; //already incremented
        myMonitor = monitorList[selectedMonitor]; //set the global pointer to the monitor we found
        myElements = monitorElementsList[selectedMonitor]; //set the global pointer to the elements for that monitor
        return self; //done
    }
    else
    {
        myMonitor = NULL;
        return nil;
    }
}

- (void)dealloc
{
    int i;
    for(i=0; i<MAX_MONITORS; i++)
    {
        if(monitorElementsList[i]!=NULL)
        {
            free((void*)monitorElementsList[i]);
        }
    }
    HIDReleaseDeviceList();
    [super dealloc]; //last
}

#pragma mark ------multiple monitor support methods-----
- (int)getNumberOfMonitors
{
    return numberOfMonitors;
}

- (int)selectedMonitor
{
    return selectedMonitor;
}

- (BOOL)changeSelectedMonitorTo:(int)value
{
    if(monitorList[value] != NULL) {
        myMonitor = monitorList[value];
        myElements = monitorElementsList[value];
        selectedMonitor=value;
        return TRUE;
    }
    return FALSE;
}

#pragma mark ----------------set values----------------
- (void)updateSettingsUsingArray:(float*)settings
            withProgressSelector:(SEL)updateProgress
                selectorWhenDone:(SEL)finish
                         ofClass:(NSObject *)callbackObject
{
    int i;
    for(i=0; i<MYELEMENTS_LENGTH; i++)
    {
        if(settings[i]!=FLT_MIN) {
            [self setHIDElement:i withValue:(float)settings[i]];
        }
        [callbackObject performSelector:updateProgress];
    }
    [callbackObject performSelector:finish];
}

- (BOOL)setBrightness:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_Brightness withValue:value];
}

- (BOOL)setContrast:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_Contrast withValue:value];
}

- (BOOL)setRedGain:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_RedGain withValue:value];
}

- (BOOL)setGreenGain:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_GreenGain withValue:value];
}

- (BOOL)setBlueGain:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_BlueGain withValue:value];
}

- (BOOL)setFocus:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_Focus withValue:value];
}

- (BOOL)setHorizontalPosition:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizPosition withValue:value];
}

- (BOOL)setHorizontalSize:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizSize withValue:value];
}

- (BOOL)setHorizontalPincushion:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizPin withValue:value];
}

- (BOOL)setHorizontalPincushionBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizPinBal withValue:value];
}

- (BOOL)setHorizontalMisconvergence:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizMisConv withValue:value];
}

- (BOOL)setHorizontalLinearity:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizLin withValue:value];
}

- (BOOL)setHorizontalLinearityBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizLinBal withValue:value];
}

- (BOOL)setVerticalPosition:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertPosition withValue:value];
}

- (BOOL)setVerticalSize:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertSize withValue:value];
}

- (BOOL)setVerticalPincushion:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertPin withValue:value];
}

- (BOOL)setVerticalPincushionBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertPinBal withValue:value];
}

- (BOOL)setVerticalMisconvergence:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertMisConv withValue:value];
}

- (BOOL)setVerticalLinearity:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertLin withValue:value];
}

- (BOOL)setVerticalLinearityBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertLinBal withValue:value];
}

- (BOOL)setKeystoneBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_KeyBalance withValue:value];
}

- (BOOL)setKeystone:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_Key withValue:value];
}

- (BOOL)setRotation:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_Tilt withValue:value];
}

- (BOOL)setTopCornerDistortionControl:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_TCDistControl withValue:value];
}

- (BOOL)setTopCornerDistortionBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_TCDistBalance withValue:value];
}

- (BOOL)setBottomCornerDistortionControl:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_BCDistControl withValue:value];
}

- (BOOL)setBottomCornerDistortionBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_BCDistBalance withValue:value];
}

- (BOOL)setHorizontalMoire:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizMoire withValue:value];
}

- (BOOL)setVerticalMoire:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertMoire withValue:value];
}

- (BOOL)setRedLuminance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_RedLuminance withValue:value];
}

- (BOOL)setGreenLuminance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_GreenLuminance withValue:value];
}

- (BOOL)setBlueLuminance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_BlueLuminance withValue:value];
}

- (BOOL)degauss
{
    return [self setHIDElement:kHIDVESAUsagePage_Degauss withValue:1];
}

#pragma mark -----check if a control is supported------
//check if the monitor supports a control
- (BOOL)hasBrightness
{
    return (myElements[ kHIDVESAUsagePage_Brightness ] != NULL);
}

- (BOOL)hasContrast
{
    return (myElements[ kHIDVESAUsagePage_Contrast ] != NULL);
}

- (BOOL)hasRedGain
{
    return (myElements[ kHIDVESAUsagePage_RedGain ] != NULL);
}

- (BOOL)hasGreenGain
{
    return (myElements[ kHIDVESAUsagePage_GreenGain ] != NULL);
}

- (BOOL)hasBlueGain
{
    return (myElements[ kHIDVESAUsagePage_BlueGain ] != NULL);
}

- (BOOL)hasFocus
{
    return (myElements[ kHIDVESAUsagePage_Focus ] != NULL);
}

- (BOOL)hasHorizontalPosition
{
    return (myElements[ kHIDVESAUsagePage_HorizPosition ] != NULL);
}

- (BOOL)hasHorizontalSize
{
    return (myElements[ kHIDVESAUsagePage_HorizSize ] != NULL);
}

- (BOOL)hasHorizontalPincushion
{
    return (myElements[ kHIDVESAUsagePage_HorizPin ] != NULL);
}

- (BOOL)hasHorizontalPincushionBalance
{
    return (myElements[ kHIDVESAUsagePage_HorizPinBal ] != NULL);
}

- (BOOL)hasHorizontalMisconvergence
{
    return (myElements[ kHIDVESAUsagePage_HorizMisConv ] != NULL);
}

- (BOOL)hasHorizontalLinearity
{
    return (myElements[ kHIDVESAUsagePage_HorizLin ] != NULL);
}

- (BOOL)hasHorizontalLinearityBalance
{
    return (myElements[ kHIDVESAUsagePage_HorizLinBal ] != NULL);
}

- (BOOL)hasVerticalPosition
{
    return (myElements[ kHIDVESAUsagePage_VertPosition ] != NULL);
}

- (BOOL)hasVerticalSize
{
    return (myElements[ kHIDVESAUsagePage_VertSize ] != NULL);
}

- (BOOL)hasVerticalPincushion
{
    return (myElements[ kHIDVESAUsagePage_VertPin ] != NULL);
}

- (BOOL)hasVerticalPincushionBalance
{
    return (myElements[ kHIDVESAUsagePage_VertPinBal ] != NULL);
}

- (BOOL)hasVerticalMisconvergence
{
    return (myElements[ kHIDVESAUsagePage_VertMisConv ] != NULL);
}

- (BOOL)hasVerticalLinearity
{
    return (myElements[ kHIDVESAUsagePage_VertLin ] != NULL);
}

- (BOOL)hasVerticalLinearityBalance
{
    return (myElements[ kHIDVESAUsagePage_VertLinBal ] != NULL);
}

- (BOOL)hasKeystoneBalance
{
    return (myElements[ kHIDVESAUsagePage_KeyBalance ] != NULL);
}

- (BOOL)hasKeystone
{
    return (myElements[ kHIDVESAUsagePage_Key ] != NULL);
}

- (BOOL)hasRotation
{
    return (myElements[ kHIDVESAUsagePage_Tilt ] != NULL);
}

- (BOOL)hasTopCornerDistortionControl
{
    return (myElements[ kHIDVESAUsagePage_TCDistControl ] != NULL);
}

- (BOOL)hasTopCornerDistortionBalance
{
    return (myElements[ kHIDVESAUsagePage_TCDistBalance ] != NULL);
}

- (BOOL)hasBottomCornerDistortionControl
{
    return (myElements[ kHIDVESAUsagePage_BCDistControl ] != NULL);
}

- (BOOL)hasBottomCornerDistortionBalance
{
    return (myElements[ kHIDVESAUsagePage_BCDistBalance ] != NULL);
}

- (BOOL)hasHorizontalMoire
{
    return (myElements[ kHIDVESAUsagePage_HorizMoire ] != NULL);
}

- (BOOL)hasVerticalMoire
{
    return (myElements[ kHIDVESAUsagePage_VertMoire ] != NULL);
}

- (BOOL)hasRedLuminance
{
    return (myElements[ kHIDVESAUsagePage_RedLuminance ] != NULL);
}

- (BOOL)hasGreenLuminance
{
    return (myElements[ kHIDVESAUsagePage_GreenLuminance ] != NULL);
}

- (BOOL)hasBlueLuminance
{
    return (myElements[ kHIDVESAUsagePage_BlueLuminance ] != NULL);
}

- (BOOL)hasDegauss
{
    return (myElements[ kHIDVESAUsagePage_Degauss ] != NULL);
}

#pragma mark ----------------get values----------------
//caller is responsible for freeing result
- (float*)settingsArray
{
    float *settings;
    int i;

    settings = (float*) malloc(sizeof(float)*MYELEMENTS_LENGTH);
    for(i=0; i<MYELEMENTS_LENGTH; i++)
    {
        if( (myElements[i] != NULL) && (i != kHIDVESAUsagePage_Degauss) ) //skip degauss usage!
        {
            settings[i] = [self getHIDElementValue: myElements[i]->usage];
        } else {
            settings[i] = FLT_MIN;
        }
    }
    return settings;
}

- (float)brightness
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_Brightness];
}

- (float)contrast
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_Contrast];
}

- (float)redGain
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_RedGain];
}

- (float)greenGain
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_GreenGain];
}

- (float)blueGain
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_BlueGain];
}

- (float)focus
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_Focus];
}

- (float)horizontalPosition
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizPosition];
}

- (float)horizontalSize
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizSize];
}

- (float)horizontalPincushion
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizPin];
}

- (float)horizontalPincushionBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizPinBal];
}

- (float)horizontalMisconvergence
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizMisConv];
}

- (float)horizontalLinearity
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizLin];
}

- (float)horizontalLinearityBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizLinBal];
}

- (float)verticalPosition
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertPosition];
}

- (float)verticalSize
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertSize];
}

- (float)verticalPincushion
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertPin];
}

- (float)verticalPincushionBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertPinBal];
}

- (float)verticalMisconvergence
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertMisConv];
}

- (float)verticalLinearity
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertLin];
}

- (float)verticalLinearityBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertLinBal];
}

- (float)keystoneBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_KeyBalance];
}

- (float)keystone
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_Key];
}

- (float)rotation
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_Tilt];
}

- (float)topCornerDistortionControl
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_TCDistControl];
}

- (float)topCornerDistortionBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_TCDistBalance];
}

- (float)bottomCornerDistortionControl
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_BCDistControl];
}

- (float)bottomCornerDistortionBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_BCDistBalance];
}

- (float)horizontalMoire
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizMoire];
}

- (float)verticalMoire
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertMoire];
}

- (float)redLuminance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_RedLuminance];
}

- (float)greenLuminance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_GreenLuminance];
}

- (float)blueLuminance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_BlueLuminance];
}



#pragma mark ----------------direct methods----------------
- (BOOL)setHIDElement:(int)key withValue:(float)value
{
    pRecElement thisElement;
    IOHIDEventStruct myEvent;

    if( (value > 100.0) || (value < 0.0) ) {
        NSLog(@"setHIDElement called with illegal value: %f, with key %d\n",value,key);
        return FALSE;
    }

    if(!(key < MYELEMENTS_LENGTH) || (key < 0)) {
        NSLog(@"setHIDElement called with illegal key: %d (value was %f)\n",key,value);
        return FALSE;
    }
    
    thisElement = myElements[key];

    if(thisElement == NULL) {
        NSLog(@"setHIDElement called with illegal key: %d (value was %f)\n",key,value);
        return FALSE;
    }

    HIDTransactionGetElementValue(myMonitor,thisElement,&myEvent);
//    NSLog(@"Got percent: %f\n",value);
    myEvent.value = (value/100.0)*(thisElement->userMax);
//    NSLog(@"Setting value: %ld\n",myEvent.value);
    HIDSetElementValue(myMonitor,thisElement,&myEvent);

    return TRUE;
}

- (BOOL)hasHIDElementWithKey:(int)key
{
    if(!(key < MYELEMENTS_LENGTH) || (key < 0)) {
        NSLog(@"setHIDElement called with illegal key: %d\n",key);
        return FALSE;
    }    
    return (myElements[key] != NULL);
}

- (float)getHIDElementValue:(int)key
{
    pRecElement thisElement;
    
    if(!(key < MYELEMENTS_LENGTH) || (key < 0)) {
        NSLog(@"setHIDElement called with illegal key: %d\n",key);
        return FLT_MIN;
    }
    
    thisElement = myElements[key];
    if(thisElement != NULL)
    {
        return (float) ((HIDGetElementValue(myMonitor,thisElement))/(float)(thisElement->userMax))*100.0;
    }
    else
    {
        return FLT_MIN;
    }
}

@end
